//----------------------------------------------------------------------------
// Solar Hot Water Controller + Home energy monitor unit (slave)
// Last revision 29 November 2009
// Licence: GNU GPL
// By Trystan Lea
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// Load Network library and create new instance
//----------------------------------------------------------------------------
#include "Network.h";
Network net;

//char array to store commands
char *cmd;

//Slave ID needs to be unique
//for each slave on your network
int slaveID = 11;

//----------------------------------------------------------------------------
// Load Energy Monitor library and create new instance
//----------------------------------------------------------------------------
#include "Emon.h"    //Load the library
EnergyMonitor emon;  //Create an instance

//----------------------------------------------------------------------------
// Load RTD library and create new instance
//----------------------------------------------------------------------------
#include "RTDModule.h"
RTDModule rtd;

//----------------------------------------------------------------------------
// Solar hot water system variables
//----------------------------------------------------------------------------
double CYLB,CYLT,COL;
int pumpState=0;

unsigned long timerA;

//----------------------------------------------------------------------------
// Setup
//----------------------------------------------------------------------------
void setup()
{
  
  net.setTriStatePins(8,7);                          //Only 7 is being used
  
  emon.setPins(4,3);                                 //Energy monitor analog pins
  emon.calibration( 1.116111611, 0.128401361, 2.3);  //Energy monitor calibration
   
  rtd.setPins(4,5,2);                                //RTD PT1000 pins 4,5 are digital pins for multiplexer, 2 is analog input
  
  //rtd.calibration( input number, scaler, offset )
  rtd.calibration(0, 0.120270927, -15.066198679);   //INPUT 0 pin 12
  rtd.calibration(1, 0.120821076, -13.824162893);   //INPUT 1 pin 14
  rtd.calibration(2, 0.120400012, -14.91327759);    //INPUT 2 pin 15
      
  analogReference(INTERNAL);                        //Set reference to 1.1V ready for RTD measurements

  pinMode(2, OUTPUT);                               //Pump pin  

  Serial.begin(9600);
}

//----------------------------------------------------------------------------
// Main loop
//----------------------------------------------------------------------------
void loop()
{
  if ((millis()-timerA)>1000)
  {
    timerA = millis();
    
    for (int i=0; i<20; i++)  //20 times for digital high pass filter
    {
      COL = digitalLowPass(COL, rtd.getTemperature(0) ,0.90);
      CYLB = digitalLowPass(CYLB, rtd.getTemperature(1) ,0.90);
      CYLT = digitalLowPass(CYLT, rtd.getTemperature(2) ,0.90);
    } 
  
    solarHotWaterControl();
   
    analogReference(DEFAULT);   //Set to 5V
    emon.calc(20,2000);         //Energy Monitor calc function
    analogReference(INTERNAL);  //Set to 1.1V
    analogRead(2);              //clear the analog channel after vref switch
  }

  //If serial line recieved
  if (net.available()) 
  {
    cmd = net.readCmd();
    //Check for standard slave commands
    SlaveCommands();           //Registers are set in here have a look in cmdList          
  } 
}
//----------------------------------------------------------------------------


